import subprocess

from watchdog.events import FileSystemEventHandler


class EventHandler(FileSystemEventHandler):
    def __init__(self, file_name):
        self.file_name = file_name
        self.processes = []

    def start_server_first_time(self):
        if self.processes:
            raise Exception("Something wrong with the server")
        self.processes.append(subprocess.Popen(["python3", self.file_name], start_new_session=False))

    def on_any_event(self, event):
        """
        This function is a callback that will start a new server on every even change

        :param event FSEvent: a data structure with info about the events
        """

        if len(self.processes) > 0:
            for process in self.processes:
                process.terminate()
        self.processes.append(subprocess.Popen(["python3", self.file_name], start_new_session=False))



from typing import Any

def static_file(file_path: str) -> dict[str, Any]:
    """
    This function will help in serving a static_file

    :param file_path str: file path to serve as a response
    """

    pass

def jsonify(input_dict: object) -> str:
    """
    This function serializes input dict to a json string

    :param input_dict object: response of the function
    """

    pass



from abc import ABC, abstractmethod
from typing import Callable, Tuple

from robyn.ws import WS

Route = Tuple[str, str, Callable, bool, int]

class BaseRouter(ABC):
    @abstractmethod
    def add_route(*args) -> None:
        pass

class Router(BaseRouter):
    def __init__(self) -> None:
        pass
    def add_route(self, route_type: str, endpoint: str, handler: Callable, const: bool) -> None:
        pass
    def get_routes(self) -> list[Route]:
        pass

class MiddlewareRouter(BaseRouter):
    def __init__(self) -> None:
        pass
    def add_route(self, route_type: str, endpoint: str, handler: Callable) -> None:
        pass
    def add_after_request(self, endpoint: str) -> Callable[..., None]:
        pass
    def add_before_request(self, endpoint: str) -> Callable[..., None]:
        pass
    def get_routes(self) -> list[Route]:
        pass

class WebSocketRouter(BaseRouter):
    def __init__(self) -> None:
        pass
    def add_route(self, endpoint: str, web_socket: WS) -> None:
        pass
    def get_routes(self) -> dict[str, WS]:
        pass



from watchdog.events import FileSystemEvent, FileSystemEventHandler

class EventHandler(FileSystemEventHandler):
    def __init__(self, file_name: str) -> None: ...
    def start_server_first_time(self) -> None: ...
    def on_any_event(self, event: FileSystemEvent) -> None:
        """
        This function is a callback that will start a new server on every even change

        :param event FSEvent: a data structure with info about the events
        """



from robyn import Robyn, static_file, jsonify, WS

from robyn.log_colors import Colors
import asyncio
import os
import pathlib
import logging

app = Robyn(__file__)
websocket = WS(app, "/web_socket")
i = -1

logger = logging.getLogger(__name__)


@websocket.on("message")
async def connect(websocket_id):
    print(websocket_id)
    global i
    i += 1
    if i == 0:
        return "Whaaat??"
    elif i == 1:
        return "Whooo??"
    elif i == 2:
        i = -1
        return "*chika* *chika* Slim Shady."


@websocket.on("close")
def close():
    return "GoodBye world, from ws"


@websocket.on("connect")
def message():
    return "Hello world, from ws"


callCount = 0


@app.get("/")
async def hello(request):
    global callCount
    callCount += 1
    message = "Called " + str(callCount) + " times"
    print(message, request)
    return {"status_code": "200", "body": "hello", "type": "text"}


@app.get("/const_request", const=True)
async def const_request():
    return "Hello world"


@app.get("/const_request_json", const=True)
async def const_request_json():
    return jsonify({"hello": "world"})


@app.get('/404')
def return_404():
    return {"status_code": "404", "body": "hello", "type": "text"}


@app.post('/404')
def return_404_post():
    return {"status_code": "404", "body": "hello", "type": "text"}


@app.before_request("/")
async def hello_before_request(request):
    global callCount
    callCount += 1
    print(request)
    return ""


@app.after_request("/")
async def hello_after_request(request):
    global callCount
    callCount += 1
    print(request)
    return ""


@app.get("/test/:id")
async def test(request):
    print(request)
    current_file_path = pathlib.Path(__file__).parent.resolve()
    html_file = os.path.join(current_file_path, "index.html")

    return static_file(html_file)


@app.get("/jsonify")
async def json_get():
    return jsonify({"hello": "world"})


@app.get("/query")
async def query_get(request):
    query_data = request["queries"]
    return jsonify(query_data)


@app.post("/jsonify/:id")
async def json(request):
    return jsonify({"hello": "world"})


@app.post("/post")
async def post():
    return "POST Request"


@app.post("/post_with_body")
async def postreq_with_body(request):
    return bytearray(request["body"]).decode("utf-8")


@app.put("/put")
async def put(request):
    return "PUT Request"


@app.put("/put_with_body")
async def putreq_with_body(request):
    print(request)
    return bytearray(request["body"]).decode("utf-8")


@app.post("/headers")
async def postreq_with_headers(request):
    logger.info(f"{Colors.OKGREEN} {request['headers']} \n{Colors.ENDC}")
    return jsonify(request["headers"])


@app.delete("/delete")
async def delete():
    return "DELETE Request"


@app.delete("/delete_with_body")
async def deletereq_with_body(request):
    return bytearray(request["body"]).decode("utf-8")


@app.patch("/patch")
async def patch():
    return "PATCH Request"


@app.patch("/patch_with_body")
async def patchreq_with_body(request):
    return bytearray(request["body"]).decode("utf-8")


@app.get("/sleep")
async def sleeper():
    await asyncio.sleep(5)
    return "sleep function"


@app.get("/blocker")
def blocker():
    import time

    time.sleep(10)
    return "blocker function"


async def startup_handler():
    print("Starting up")


@app.shutdown_handler
def shutdown_handler():
    print("Shutting down")


@app.get("/redirect")
async def redirect(request):
    return {"status_code": "307", "body": "", "type": "text", "headers": jsonify({"Location": "redirect_route"})}


@app.get("/redirect_route")
async def redirect_route(request):
    return "This is the redirected route"


if __name__ == "__main__":
    ROBYN_URL = os.getenv("ROBYN_URL", "0.0.0.0")
    ROBYN_PORT = int(os.getenv("ROBYN_PORT", "5000"))
    app.add_header("server", "robyn")
    current_file_path = pathlib.Path(__file__).parent.resolve()
    app.add_directory(
        route="/test_dir",
        directory_path=os.path.join(current_file_path, "build"),
        index_file="index.html",
    )
    app.startup_handler(startup_handler)
    app.start(port=ROBYN_PORT, url=ROBYN_URL)



from numbers import Number
from typing import Dict, Tuple

from robyn.events import Events
from robyn.robyn import SocketHeld
from robyn.router import Route
from robyn.ws import WS

Directory = Tuple[str, str, str, str]
Header = Tuple[str, str]

def spawn_process(
    directories: Tuple[Directory, ...],
    headers: Tuple[Header, ...],
    routes: Tuple[Route, ...],
    middlewares: Tuple[Route, ...],
    web_sockets: Dict[str, WS],
    event_handlers: Dict[Events, list],
    socket: SocketHeld,
    workers: Number,
) -> None:
    """
    This function is called by the main process handler to create a server runtime.
    This functions allows one runtime per process.

    :param directories tuple: the list of all the directories and related data in a tuple
    :param headers tuple: All the global headers in a tuple
    :param routes Tuple[Route]: The routes touple, containing the description about every route.
    :param middlewares Tuple[Route]: The middleware router touple, containing the description about every route.
    :param web_sockets list: This is a list of all the web socket routes
    :param event_handlers Dict: This is an event dict that contains the event handlers
    :param socket SocketHeld: This is the main tcp socket, which is being shared across multiple processes.
    :param process_name string: This is the name given to the process to identify the process
    :param workers number: This is the name given to the process to identify the process
    """



import asyncio
import multiprocessing as mp
import os
from inspect import signature
from typing import Callable

from multiprocess import Process
from watchdog.observers import Observer

from robyn.events import Events

from .argument_parser import ArgumentParser
from .dev_event_handler import EventHandler
from .log_colors import Colors
from .processpool import spawn_process
from .responses import jsonify, static_file
from .robyn import Server, SocketHeld
from .router import MiddlewareRouter, Router, WebSocketRouter
from .ws import WS


class Robyn:
    """This is the python wrapper for the Robyn binaries."""

    def __init__(self, file_object: str) -> None: ...
    def before_request(self, endpoint: str) -> Callable[..., None]:
        """
        The @app.before_request decorator to add a get route

        :param endpoint str: endpoint to server the route
        """
        ...

    def after_request(self, endpoint: str) -> Callable[..., None]:
        """
        The @app.after_request decorator to add a get route

        :param endpoint str: endpoint to server the route
        """
        ...

    def add_directory(
        self,
        route: str,
        directory_path: str,
        index_file: str = ...,
        show_files_listing: str = ...,
    ): ...
    def add_header(self, key: str, value: str) -> None: ...
    def add_web_socket(self, endpoint: str, ws: WS) -> None: ...
    def startup_handler(self, handler: Callable) -> None: ...
    def shutdown_handler(self, handler: Callable) -> None: ...
    def start(self, url: str = ..., port: int = ...) -> None:
        """
        Starts the server

        :param port int: reperesents the port number at which the server is listening
        """

        ...
    def get(self, endpoint: str, const: bool = False) -> Callable[..., None]:
        """
        The @app.get decorator to add a get route

        :param endpoint str: endpoint to server the route
        """
        ...
    def post(self, endpoint: str) -> Callable[..., None]:
        """
        The @app.post decorator to add a get route

        :param endpoint str: endpoint to server the route
        """
        ...
    def put(self, endpoint: str) -> Callable[..., None]:
        """
        The @app.put decorator to add a get route

        :param endpoint str: endpoint to server the route
        """

        ...
    def delete(self, endpoint: str) -> Callable[..., None]:
        """
        The @app.delete decorator to add a get route

        :param endpoint str: endpoint to server the route
        """

        ...
    def patch(self, endpoint: str) -> Callable[..., None]:
        """
        The @app.patch decorator to add a get route

        :param endpoint str: endpoint to server the route
        """

        ...

    def head(self, endpoint: str) -> Callable[..., None]:
        """
        The @app.head decorator to add a get route

        :param endpoint str: endpoint to server the route
        """

        ...
    def options(self, endpoint: str) -> Callable[..., None]:
        """
        The @app.options decorator to add a get route

        :param endpoint str: endpoint to server the route
        """

        ...
    def connect(self, endpoint: str) -> Callable[..., None]:
        """
        The @app.connect decorator to add a get route

        :param endpoint str: endpoint to server the route
        """
        ...
    def trace(self, endpoint: str) -> Callable[..., None]:
        """
        The @app.trace decorator to add a get route

        :param endpoint str: endpoint to server the route
        """

        ...



import requests

BASE_URL = "http://127.0.0.1:5000"


def test_404_status_code(session):
    res = requests.get(f"{BASE_URL}/404")
    assert res.status_code == 404


def test_404_post_request_status_code(session):
    r = requests.post(f"{BASE_URL}/404")
    assert r.status_code == 404


def test_307_get_request(session):
    r = requests.get(f"{BASE_URL}/redirect")
    assert r.text == "This is the redirected route"




from robyn import Robyn

class WS:
    """This is the python wrapper for the web socket that will be used here."""

    def __init__(self, robyn_object: Robyn, endpoint: str) -> None:
        pass
    def on(self, type: str) -> None:
        pass



from __future__ import annotations

from typing import Callable, Optional, Tuple

class SocketHeld:
    def __init__(self, url: str, port: int):
        pass
    def try_clone(self) -> SocketHeld:
        pass

class Server:
    def __init__(self) -> None:
        pass
    def add_directory(
        self,
        route: str,
        directory_path: str,
        index_file: Optional[str],
        show_files_listing: bool,
    ) -> None:
        pass
    def add_header(self, key: str, value: str) -> None:
        pass
    def add_route(
        self,
        route_type: str,
        route: str,
        handler: Callable,
        is_async: bool,
        number_of_params: int,
    ) -> None:
        pass
    def add_middleware_route(
        self,
        route_type: str,
        route: str,
        handler: Callable,
        is_async: bool,
        number_of_params: int,
    ) -> None:
        pass
    def add_startup_handler(self, handler: Callable, is_async: bool) -> None:
        pass
    def add_shutdown_handler(self, handler: Callable, is_async: bool) -> None:
        pass
    def add_web_socket_route(
        self,
        route: str,
        connect_route: Tuple[Callable, bool, int],
        close_route: Tuple[Callable, bool, int],
        message_route: Tuple[Callable, bool, int],
    ) -> None:
        pass
    def start(self, socket: SocketHeld, workers: int) -> None:
        pass



import argparse


class ArgumentParser(argparse.ArgumentParser):
    def __init__(self) -> None:
        ...

    @property
    def num_processes(self) -> int:
        ...

    @property
    def workers(self) -> int:
        ...

    @property
    def is_dev(self) -> bool:
        ...

    @property
    def log_level(self) -> str:
        ...




from abc import ABC, abstractmethod
from asyncio import iscoroutinefunction
from inspect import signature


class BaseRouter(ABC):
    @abstractmethod
    def add_route(*args):
        ...


class Router(BaseRouter):
    def __init__(self) -> None:
        super().__init__()
        self.routes = []

    def _format_response(self, res):
        # handle file handlers
        response = {}
        if type(res) == dict:
            if "status_code" not in res:
                res["status_code"] = "200"
                response = res
            else:
                response = {
                    "status_code": "200",
                    "body": res["body"],
                    **res
                }
        else:
            response = {"status_code": "200", "body": res, "type": "text"}

        return response

    def add_route(self, route_type, endpoint, handler, const):
        async def async_inner_handler(*args):
            response = self._format_response(await handler(*args))
            return response

        def inner_handler(*args):
            response = self._format_response(handler(*args))
            return response

        number_of_params = len(signature(handler).parameters)
        if iscoroutinefunction(handler):
            self.routes.append(
                (
                    route_type,
                    endpoint,
                    async_inner_handler,
                    True,
                    number_of_params,
                    const
                )
            )
        else:
            self.routes.append(
                (
                    route_type,
                    endpoint,
                    inner_handler,
                    False,
                    number_of_params,
                    const
                )
            )

    def get_routes(self):
        return self.routes


class MiddlewareRouter(BaseRouter):
    def __init__(self) -> None:
        super().__init__()
        self.routes = []

    def add_route(self, route_type, endpoint, handler):
        number_of_params = len(signature(handler).parameters)
        self.routes.append(
            (
                route_type,
                endpoint,
                handler,
                iscoroutinefunction(handler),
                number_of_params,
            )
        )

    # These inner function is basically a wrapper arround the closure(decorator)
    # being returned.
    # It takes in a handler and converts it in into a closure
    # and returns the arguments.
    # Arguments are returned as they could be modified by the middlewares.
    def add_after_request(self, endpoint):
        def inner(handler):
            async def async_inner_handler(*args):
                await handler(*args)
                return args

            def inner_handler(*args):
                handler(*args)
                return args

            if iscoroutinefunction(handler):
                self.add_route("AFTER_REQUEST", endpoint, async_inner_handler)
            else:
                self.add_route("AFTER_REQUEST", endpoint, inner_handler)

        return inner

    def add_before_request(self, endpoint):
        def inner(handler):
            async def async_inner_handler(*args):
                await handler(*args)
                return args

            def inner_handler(*args):
                handler(*args)
                return args

            if iscoroutinefunction(handler):
                self.add_route("BEFORE_REQUEST", endpoint, async_inner_handler)
            else:
                self.add_route("BEFORE_REQUEST", endpoint, inner_handler)

        return inner

    def get_routes(self):
        return self.routes


class WebSocketRouter(BaseRouter):
    def __init__(self) -> None:
        super().__init__()
        self.routes = {}

    def add_route(self, endpoint, web_socket):
        self.routes[endpoint] = web_socket

    def get_routes(self):
        return self.routes



from websockets import connect
import asyncio

BASE_URL = "ws://127.0.0.1:5000"

def test_web_socket(session):
    async def start_ws(uri):
        async with connect(uri) as websocket:
            assert( await websocket.recv() == "Hello world, from ws")
            await websocket.send("My name is?")
            assert( await websocket.recv() == "Whaaat??")
            await websocket.send("My name is?")
            assert( await websocket.recv() == "Whooo??")
            await websocket.send("My name is?")
            assert( await websocket.recv() == "*chika* *chika* Slim Shady.")

    asyncio.run(start_ws(f"{BASE_URL}/web_socket"))





import asyncio
import logging
import multiprocessing as mp
import os

from multiprocess import Process
from watchdog.observers import Observer
from robyn.events import Events
from .argument_parser import ArgumentParser
from .dev_event_handler import EventHandler
from .log_colors import Colors
from .processpool import spawn_process
from .responses import jsonify, static_file

from .robyn import SocketHeld
from .router import MiddlewareRouter, Router, WebSocketRouter
from .ws import WS

mp.allow_connection_pickling()

logger = logging.getLogger(__name__)


class Robyn:
    """This is the python wrapper for the Robyn binaries."""

    def __init__(self, file_object):
        directory_path = os.path.dirname(os.path.abspath(file_object))
        self.file_path = file_object
        self.directory_path = directory_path
        self.parser = ArgumentParser()
        self.dev = self.parser.is_dev
        self.processes = self.parser.num_processes
        self.workers = self.parser.workers
        self.log_level = self.parser.log_level
        self.router = Router()
        self.middleware_router = MiddlewareRouter()
        self.web_socket_router = WebSocketRouter()
        self.headers = []
        self.directories = []
        self.event_handlers = {}

        self._config_logger()

    def _add_route(self, route_type, endpoint, handler, const=False):
        """
        [This is base handler for all the decorators]

        :param route_type [str]: [route type between GET/POST/PUT/DELETE/PATCH]
        :param endpoint [str]: [endpoint for the route added]
        :param handler [function]: [represents the sync or async function passed as a handler for the route]
        """

        """ We will add the status code here only
        """
        self.router.add_route(route_type, endpoint, handler, const)

    def before_request(self, endpoint):
        """
        The @app.before_request decorator to add a get route

        :param endpoint str: endpoint to server the route
        """

        return self.middleware_router.add_before_request(endpoint)

    def after_request(self, endpoint):
        """
        The @app.after_request decorator to add a get route

        :param endpoint str: endpoint to server the route
        """

        return self.middleware_router.add_after_request(endpoint)

    def add_directory(
        self, route, directory_path, index_file=None, show_files_listing=False
    ):
        self.directories.append((route, directory_path, index_file, show_files_listing))

    def add_header(self, key, value):
        self.headers.append((key, value))

    def add_web_socket(self, endpoint, ws):
        self.web_socket_router.add_route(endpoint, ws)

    def _add_event_handler(self, event_type: str, handler):
        logger.debug(f"Add event {event_type} handler")
        if event_type not in {Events.STARTUP, Events.SHUTDOWN}:
            return

        is_async = asyncio.iscoroutinefunction(handler)
        self.event_handlers[event_type] = (handler, is_async)

    def startup_handler(self, handler):
        self._add_event_handler(Events.STARTUP, handler)

    def shutdown_handler(self, handler):
        self._add_event_handler(Events.SHUTDOWN, handler)

    def start(self, url="127.0.0.1", port=5000):
        """
        Starts the server

        :param port int: reperesents the port number at which the server is listening
        """

        if not self.dev:
            processes = []
            workers = self.workers
            socket = SocketHeld(url, port)
            for _ in range(self.processes):
                copied_socket = socket.try_clone()
                p = Process(
                    target=spawn_process,
                    args=(
                        self.directories,
                        self.headers,
                        self.router.get_routes(),
                        self.middleware_router.get_routes(),
                        self.web_socket_router.get_routes(),
                        self.event_handlers,
                        copied_socket,
                        workers,
                    ),
                )
                p.start()
                processes.append(p)

            logger.info(f"{Colors.HEADER}Starting up \n{Colors.ENDC}")
            logger.info(f"{Colors.OKGREEN}Press Ctrl + C to stop \n{Colors.ENDC}")
            try:
                for process in processes:
                    process.join()
            except KeyboardInterrupt:
                logger.info(f"{Colors.BOLD}{Colors.OKGREEN} Terminating server!! {Colors.ENDC}")
                for process in processes:
                    process.kill()
        else:
            event_handler = EventHandler(self.file_path)
            event_handler.start_server_first_time()
            logger.info(
                f"{Colors.OKBLUE}Dev server initialised with the directory_path : {self.directory_path}{Colors.ENDC}"
            )
            observer = Observer()
            observer.schedule(event_handler, path=self.directory_path, recursive=True)
            observer.start()
            try:
                while True:
                    pass
            finally:
                observer.stop()
                observer.join()

    def get(self, endpoint, const=False):
        """
        The @app.get decorator to add a get route

        :param endpoint str: endpoint to server the route
        """

        def inner(handler):
            self._add_route("GET", endpoint, handler, const)

        return inner

    def post(self, endpoint):
        """
        The @app.post decorator to add a get route

        :param endpoint str: endpoint to server the route
        """

        def inner(handler):
            self._add_route("POST", endpoint, handler)

        return inner

    def put(self, endpoint):
        """
        The @app.put decorator to add a get route

        :param endpoint str: endpoint to server the route
        """
        def inner(handler):
            self._add_route("PUT", endpoint, handler)

        return inner

    def delete(self, endpoint):
        """
        The @app.delete decorator to add a get route

        :param endpoint str: endpoint to server the route
        """

        def inner(handler):
            self._add_route("DELETE", endpoint, handler)

        return inner

    def patch(self, endpoint):
        """
        [The @app.patch decorator to add a get route]

        :param endpoint [str]: [endpoint to server the route]
        """

        def inner(handler):
            self._add_route("PATCH", endpoint, handler)

        return inner

    def head(self, endpoint):
        """
        The @app.head decorator to add a get route

        :param endpoint str: endpoint to server the route
        """

        def inner(handler):
            self._add_route("HEAD", endpoint, handler)

        return inner

    def options(self, endpoint):
        """
        The @app.options decorator to add a get route

        :param endpoint str: endpoint to server the route
        """

        def inner(handler):
            self._add_route("OPTIONS", endpoint, handler)

        return inner

    def connect(self, endpoint):
        """
        The @app.connect decorator to add a get route

        :param endpoint str: endpoint to server the route
        """

        def inner(handler):
            self._add_route("CONNECT", endpoint, handler)

        return inner

    def trace(self, endpoint):
        """
        The @app.trace decorator to add a get route

        :param endpoint str: endpoint to server the route
        """

        def inner(handler):
            self._add_route("TRACE", endpoint, handler)

        return inner

    def _config_logger(self):
        """
        This is the method to configure the logger either on the dev mode or the env variable
        """

        log_level = "WARN"

        if self.dev:
            log_level = "DEBUG"

        log_level = self.log_level if self.log_level else log_level
        logging.basicConfig(level=log_level)




import asyncio
import logging
import multiprocessing as mp
import sys

from .events import Events
from .robyn import Server


mp.allow_connection_pickling()


def spawn_process(
    directories, headers, routes, middlewares, web_sockets, event_handlers, socket, workers
):
    """
    This function is called by the main process handler to create a server runtime.
    This functions allows one runtime per process.

    :param directories tuple: the list of all the directories and related data in a tuple
    :param headers tuple: All the global headers in a tuple
    :param routes Tuple[Route]: The routes touple, containing the description about every route.
    :param middlewares Tuple[Route]: The middleware router touple, containing the description about every route.
    :param web_sockets list: This is a list of all the web socket routes
    :param event_handlers Dict: This is an event dict that contains the event handlers
    :param socket SocketHeld: This is the main tcp socket, which is being shared across multiple processes.
    :param process_name string: This is the name given to the process to identify the process
    :param workers number: This is the name given to the process to identify the process
    """

    # platform_name = platform.machine()
    if sys.platform.startswith("win32") or sys.platform.startswith("linux-cross"):
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
    else:
        # uv loop doesn't support windows or arm machines at the moment
        # but uv loop is much faster than native asyncio
        import uvloop

        uvloop.install()
        loop = uvloop.new_event_loop()
        asyncio.set_event_loop(loop)

    server = Server()

    # TODO: if we remove the dot access
    # the startup time will improve in the server
    for directory in directories:
        route, directory_path, index_file, show_files_listing = directory
        server.add_directory(route, directory_path, index_file, show_files_listing)

    for key, val in headers:
        server.add_header(key, val)

    for route in routes:
        route_type, endpoint, handler, is_async, number_of_params, const = route
        server.add_route(route_type, endpoint, handler, is_async, number_of_params, const)

    for route in middlewares:
        route_type, endpoint, handler, is_async, number_of_params = route
        server.add_middleware_route(route_type, endpoint, handler, is_async, number_of_params)

    if "startup" in event_handlers:
        server.add_startup_handler(event_handlers[Events.STARTUP][0], event_handlers[Events.STARTUP][1])

    if "shutdown" in event_handlers:
        server.add_shutdown_handler(event_handlers[Events.SHUTDOWN][0], event_handlers[Events.SHUTDOWN][1])

    for endpoint in web_sockets:
        web_socket = web_sockets[endpoint]
        server.add_web_socket_route(
            endpoint,
            web_socket.methods["connect"],
            web_socket.methods["close"],
            web_socket.methods["message"],
        )

    try:
        server.start(socket, workers)
        loop = asyncio.get_event_loop()
        loop.run_forever()
    except KeyboardInterrupt:
        loop.close()



import json


def static_file(file_path):
    """
    This function will help in serving a static_file

    :param file_path str: file path to serve as a response
    """

    return {
        "type": "static_file",
        "file_path": file_path,
        # this is a hack for now
        "body": "",
        "status_code": "200",
    }


def jsonify(input_dict):
    """
    This function serializes input dict to a json string

    :param input_dict dict: response of the function
    """

    return json.dumps(input_dict)

